from grokui.base.interfaces import IGrokUIRealm, IUIPanel, IMainMenu
from grokui.base.namespace import GrokUILayer, GrokUISkin, GrokUINamespace
from grokui.base.contentproviders import Header, Footer, MainMenu, Messages
from grokui.base.layout import GrokUIView



# -*- coding: utf-8 -*-

import grokcore.viewlet as grok
from grokui.base import Header, Footer, Messages, IUIPanel, IGrokUIRealm
from grokcore.message.utils import receive
from zope.browsermenu.interfaces import IBrowserMenu
from zope.authentication.interfaces import IUnauthenticatedPrincipal
from zope.component import getUtility

grok.view(IUIPanel)
grok.context(IGrokUIRealm)
grok.templatedir("templates")


class Banner(grok.Viewlet):
    grok.order(10)
    grok.name('grokui.banner')
    grok.viewletmanager(Header)


class LoginInformation(grok.Viewlet):
    grok.order(20)
    grok.name('grokui.login')
    grok.viewletmanager(Header)

    @property
    def is_authenticated(self):
        """Check, wether we are authenticated.
        """
        return not IUnauthenticatedPrincipal.providedBy(self.request.principal)


class MenuViewlet(grok.Viewlet):
    grok.order(30)
    grok.name("grokui.menu")
    grok.viewletmanager(Header)

    def update(self):
        menu = getUtility(IBrowserMenu, "grokui_mainmenu")
        self.rooturl = self.view.url(self.context)
        self.actions = menu.getMenuItems(self.context, self.request)


class StatusMessages(grok.Viewlet):
    grok.order(40)
    grok.name('grokui.messages')
    grok.viewletmanager(Messages)

    @property
    def messages(self):
        return receive()


class Authors(grok.Viewlet):
    grok.order(10)
    grok.name('grokui.authors')
    grok.viewletmanager(Footer)



# -*- coding: utf-8 -*-
"""Components to keep Grok UI related stuff in a dedicated namespace.
"""
import grokcore.view as grok

from zope.site.interfaces import IRootFolder
from zope.location import LocationProxy
from zope.publisher.browser import applySkin
from zope.publisher.interfaces import browser
from zope.traversing.interfaces import ITraversable
from grokui.base.interfaces import IGrokUIRealm


class GrokUILayer(grok.IDefaultBrowserLayer):
    """A basic layer for all Grok UI components.
    """
    pass


class GrokUISkin(GrokUILayer, browser.IBrowserSkinType):
    """A skin for all Grok UI stuff.
    """
    grok.skin('GrokUISkin')


class GrokUINamespace(grok.MultiAdapter):
    grok.name('grokui')
    grok.provides(ITraversable)
    grok.implements(IGrokUIRealm)
    grok.adapts(IRootFolder, browser.IBrowserRequest)

    def __init__(self, context, request):
        self.root = context
        self.request = request
        applySkin(self.request, GrokUISkin)

    def traverse(self, name, ignore):
        return LocationProxy(self, self.root, "++grokui++")



"""
Building panels using `GrokUIView`
===================================

We create a browser to watch our views:

    >>> from zope.app.wsgi.testlayer import Browser
    >>> browser = Browser()
    >>> browser.addHeader('Authorization', 'Basic mgr:mgrpw')
    >>> browser.handleErrors = False

To create a view that automatically comes with the GrokUI layout, we
can derive from `grokui.base.AdminView`.

Instances of `AdminView` are in fact `megrok.layout.Page` instances
that render the content provided by a template or `render` method
into a given layout.

When we render PlainAdminView, we will get a complete Grok UI page
with the contents delivered by the `render()` method inserted:

    >>> browser.open('http://localhost/++grokui++/caveview')
    >>> print browser.contents
    <html xmlns="http://www.w3.org/1999/xhtml">
    ...
     <title>Grok User Interface</title>
    ...
    <BLANKLINE>
        <div id="grokui-content">Hello from CaveAdminView</div>
    <BLANKLINE>
    ...
    </html>

We also provided a ``title`` with our page. Therefore we will get an
entry in the navigation bar:

    >>> print browser.contents
    <html xmlns="http://www.w3.org/1999/xhtml">
    ...
    <ul id="grokui-menu-entries">
      <li>
        <a href="http://localhost/++grokui++/caveview"
           title="cave management">cave management</a>
      </li>
    </ul>
    ...
    </html>

"""
import grokui.base
import unittest
import doctest
import grokcore.component as grok
from grokui.base import GrokUIView
from zope.fanstatic.testing import ZopeFanstaticBrowserLayer


class CaveAdminView(GrokUIView):
    """An admin page to administer caves.
    """
    grok.name('caveview')
    # This title will appear in the navigation bar:
    grok.title('cave management')

    def render(self):
        """This will go into a standard Grok UI page.
        """
        return u'Hello from CaveAdminView'


def test_suite():
    suite = unittest.TestSuite()
    test = doctest.DocTestSuite(
        optionflags=doctest.ELLIPSIS + doctest.NORMALIZE_WHITESPACE)
    test.layer = ZopeFanstaticBrowserLayer(grokui.base)
    suite.addTest(test)
    return suite



import os
from setuptools import setup, find_packages

tests_require = [
    'martian',
    'zope.annotation',
    'zope.app.appsetup',
    'zope.app.pagetemplate',
    'zope.app.publication',
    'zope.app.wsgi >= 3.10.0',
    'zope.browserpage',
    'zope.browserresource',
    'zope.container',
    'zope.contentprovider',
    'zope.login',
    'zope.password',
    'zope.principalregistry',
    'zope.security',
    'zope.securitypolicy',
    ]

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

setup(name='grokui.base',
      version='0.5.1',
      description="The Grok administration and development UI (base)",
      long_description=(
        read('README.txt') +
        '\n\n' +
        read(os.path.join('src', 'grokui', 'base', 'README.txt')) +
        '\n\n' +
        read('CHANGES.txt')
        ),
      # Get strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Framework :: Zope3'],
      keywords="zope3 grok grokadmin",
      author="Zope Foundation and Contributors",
      author_email="grok-dev@zope.org",
      url="http://svn.zope.org/grokui.base",
      license="ZPL 2.1",
      package_dir={'': 'src'},
      packages=find_packages('src'),
      include_package_data=True,
      zip_safe=False,
      namespace_packages = ['grokui'],
      install_requires=[
          'fanstatic',
          'grokcore.component',
          'grokcore.message',
          'grokcore.view',
          'grokcore.viewlet',
          'megrok.layout >= 1.2',
          'megrok.menu',
          'setuptools',
          'zope.authentication',
          'zope.browsermenu',
          'zope.component',
          'zope.fanstatic',
          'zope.interface',
          'zope.location',
          'zope.publisher',
          'zope.site',
          'zope.traversing',
          ],
      tests_require = tests_require,
      extras_require = dict(test=tests_require),
      entry_points={
          'fanstatic.libraries': [
              'grokui.base = grokui.base.resource:library']}
      )



# -*- coding: utf-8 -*-

import grokcore.viewlet as grok
from megrok.menu import Menu
from zope.site.interfaces import IRootFolder
from zope.component import getUtility, getMultiAdapter
from zope.browsermenu.interfaces import IBrowserMenu
from grokui.base import IGrokUIRealm, GrokUILayer

grok.layer(GrokUILayer)
grok.context(IGrokUIRealm)


class Header(grok.ViewletManager):
    grok.name('grokui_header')


class Footer(grok.ViewletManager):
    grok.name('grokui_footer')


class Messages(grok.ViewletManager):
    grok.name('grokui_messages')


class MainMenu(Menu):
    grok.name('grokui_mainmenu')
    grok.title('Grok user interface panels')


class Index(grok.View):
    """Redirect to the grokui namespace.

    Redirect to the first item displayed in grokui-namespaced main
    menu.
    """
    grok.name('index')
    grok.context(IRootFolder)
    grok.require('zope.ManageServices')
    grok.layer(grok.IDefaultBrowserLayer)

    def render(self):
        menu = getUtility(IBrowserMenu, 'grokui_mainmenu')
        realm = getMultiAdapter((self.context, self.request),
                                name='grokui')
        items = menu.getMenuItems(realm, self.request)
        if len(items) == 0:
            # No grokui panel installed.
            return u'No further grokui components are installed.'
        first_name = items[0]['action']
        grokui_url = self.url(self.context) + '/++grokui++/@@' + first_name
        self.redirect(grokui_url)
        return
